// FastestFileCopy.java
package com.jdojo.chapter10;

import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.nio.channels.FileChannel;

public class FastestFileCopy {
	public static void main(String[] args) {
		String source = "luci5.txt";
		String sink = "luci5_copy.txt";
		try {
			copy(source, sink, false);
			System.out.println(source + " has been copied to " + sink);
		}
		catch (IOException e) {
			System.out.println(e.getMessage());
		}
	}

	public static void copy(String source, String sink,
						boolean overwrite) throws IOException {
		String msg = "";

		// Perform some error checks
		File sourceFile = new File(source);
		if (!sourceFile.exists()) {
			msg = "Source file " + sourceFile.getAbsolutePath() + 
			      " does not exist.";
			throw new IOException(msg);
		}

		File sinkFile = new File(sink);
		if (sinkFile.exists() && !overwrite) {
			msg = "File " + sinkFile.getAbsolutePath() + 
			      " already exists.";
			throw new IOException(msg);
		}

		FileChannel srcChannel = null;
		FileChannel sinkChannel = null;

		try {
			srcChannel  = new FileInputStream(sourceFile).getChannel();
			sinkChannel =	new FileOutputStream(sinkFile).getChannel();

			// Copy source file contents to the sink file 
			srcChannel.transferTo(0, srcChannel.size(), sinkChannel);
		}
		finally {
			if (sinkChannel != null) {
				sinkChannel.close();
			}

			if (srcChannel != null) {
				srcChannel.close();
			}
		}
	}
}
