// AsyncFileWriteFuture.java
package com.jdojo.chapter11;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.nio.ByteBuffer;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.charset.Charset;
import java.nio.channels.AsynchronousFileChannel;
import static java.nio.file.StandardOpenOption.WRITE;
import static java.nio.file.StandardOpenOption.CREATE;

public class AsyncFileWriteFuture {	
	public static void main(String[] args) {
		Path path = Paths.get("C:\\poems\\rainbow.txt");

		try (AsynchronousFileChannel afc = 
				AsynchronousFileChannel.open(path, WRITE, CREATE);){	

			// Get the data to write in a ByteBuffer
			ByteBuffer dataBuffer = getDataBuffer();
						
			// Perform the asynchronous write operation
			Future<Integer> result = afc.write(dataBuffer, 0);
			
			// Keep polling to see if I/O has finished
			while (!result.isDone()) { 
				try {
					// Let the thread sleep for 2 seconds 
					// before the next polling
					System.out.println("Sleeping for two" + 
					                   " seconds...");
					Thread.sleep(2000);				
				} 
				catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			
			// I/O is complete
			try {
				int writtenBytes = result.get();
				System.out.format("%s bytes written to %s%n", 
				                  writtenBytes, path);				
			} 
			catch (InterruptedException | ExecutionException e) {
				e.printStackTrace();
			} 			
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static ByteBuffer getDataBuffer() {	
		String lineSeparator = System.getProperty("line.separator");
				
		StringBuilder sb = new StringBuilder();
		sb.append("My heart leaps up when I behold");
		sb.append(lineSeparator);
		sb.append("A Rainbow in the sky");
		sb.append(lineSeparator);
		sb.append(lineSeparator);
		sb.append("So was it when my life began;");
		sb.append(lineSeparator);
		sb.append("So is it now I am a man;");
		sb.append(lineSeparator);
		sb.append("So be it when I shall grow old,");
		sb.append(lineSeparator);
		sb.append("Or let me die!");
		sb.append(lineSeparator);
		sb.append(lineSeparator);
		sb.append("The Child is father of the man;");
		sb.append(lineSeparator);
		sb.append("And I could wish my days to be");
		
		String str = sb.toString();
		Charset cs = Charset.forName("UTF-8");
		ByteBuffer bb = ByteBuffer.wrap(str.getBytes(cs));
		
		return bb;
	}
}
