// DeleteDirectoryTest.java
package com.jdojo.chapter11;

import java.io.IOException;
import java.nio.file.FileVisitor;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.attribute.BasicFileAttributes;
import static java.nio.file.FileVisitResult.CONTINUE;
import static java.nio.file.FileVisitResult.TERMINATE;

public class DeleteDirectoryTest {
	public static void main(String[] args) {
		Path dirToDelete = Paths.get("C:\\poems\\twinkle");
		FileVisitor<Path> visitor = getFileVisitor();

		try {
			Files.walkFileTree(dirToDelete, visitor);
		} 
		catch (IOException e) {
			System.out.println(e.getMessage());
		}
	}

	public static FileVisitor<Path> getFileVisitor() {
		FileVisitor<Path> visitor =
			new SimpleFileVisitor<Path>() {
				@Override
				public FileVisitResult postVisitDirectory(Path dir, 
				                  IOException e) throws IOException {	
					FileVisitResult result = CONTINUE;

					// Now, delete the directory at the end
					if (e != null) {
						System.out.format("Error deleting %s. %s%n",
						                   dir, e.getMessage());
						result = TERMINATE;
					}
					else {
						Files.delete(dir);
						System.out.format("Deleted directory %s%n", 
						                  dir);
					}
					return result;
				}

				@Override
				public FileVisitResult visitFile(Path file, 
				       BasicFileAttributes attrs) throws IOException {

					// Delete the file that we are visiting
					Files.delete(file);

					System.out.format("Deleted file %s%n", file);
					return CONTINUE;
				}
			};

		return visitor;
	}
}
