// WalkFileTreeTest.java
package com.jdojo.chapter11;

import java.io.IOException;
import java.nio.file.FileVisitor;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;
import java.nio.file.FileVisitResult;
import static java.nio.file.FileVisitResult.CONTINUE;
import java.nio.file.Files;
import java.nio.file.Paths;

public class WalkFileTreeTest {
	public static void main(String[] args) {
		Path startDir = Paths.get("C:\\poems");
		FileVisitor<Path> visitor = getFileVisitor();
		
		try {
			Files.walkFileTree(startDir, visitor);
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static FileVisitor<Path> getFileVisitor() {
		FileVisitor<Path> visitor = 
			new SimpleFileVisitor<Path>() {
				@Override
				public FileVisitResult preVisitDirectory(Path dir, 
				                         BasicFileAttributes attrs) {
					System.out.format("%s [Directory]%n", dir);
					return CONTINUE;
				}
										
				@Override
				public FileVisitResult visitFile(Path file, 
					                     BasicFileAttributes attrs) {
					System.out.format("%s [File, Size: %s bytes]%n", 
					                  file, attrs.size());
					return CONTINUE;
				}	
			};
		
		return visitor;
	}
}
