// DelayedJob.java
package com.jdojo.chapter12;

import java.util.concurrent.Delayed;
import java.util.concurrent.TimeUnit;
import java.util.Date;

public class DelayedJob implements Delayed {
	private long scheduledTime; // in milliseconds
	String jobName;

	public DelayedJob(String jobName, long scheduledTime) {
		this.scheduledTime = scheduledTime;
		this.jobName = jobName;
	}

	public long getDelay(TimeUnit unit) {
		// Positive delay means it should stay in queue.
		// Zero or negative delay means ready to be removed
		// from the queue
		long delay = this.scheduledTime - System.currentTimeMillis();
		long returnValue = unit.convert(delay, TimeUnit.MILLISECONDS);

		return returnValue;
	}

	public int compareTo(Delayed job) {
		long currentJobDelay = this.getDelay(TimeUnit.MILLISECONDS);
		long jobDelay = job.getDelay(TimeUnit.MILLISECONDS);

		int diff = 0;
		if (currentJobDelay > jobDelay) {
			diff = 1;
		} 
		else	if (currentJobDelay < jobDelay) {
				diff = -1;
		}
		return diff;
	}

	public String toString() {
		String str = "(" + this.jobName + ", " + 
		             "Scheduled Time: " + 
		             new Date(this.scheduledTime) + 
		             ")";
		return str;
	}
}
