// ClassReflection.java
package com.jdojo.chapter5;

public class ClassReflection {
	public static void main(String[] args) {
		String className = "com.jdojo.chapter5.Person";

		try {
			Class c = Class.forName(className);
			String classDesciption = getClassDescription(c);
			System.out.println(classDesciption);
		}
		catch (ClassNotFoundException e) {
			System.out.println("Exception:" + e.getMessage());
		}
	}

	public static String getClassDescription(Class c) {
		StringBuffer classDesc = new StringBuffer();

		// Remove package name from class name to get simple name
		String simpleName = c.getSimpleName();

		// Get class modifiers
		int mod = c.getModifiers();

		// Convert modifiers to their string represenation
		String modifiers = java.lang.reflect.Modifier.toString(mod);

		// Start with modifiers
		classDesc = classDesc.append(modifiers);

		// Add keyword class or interface
		if (c.isPrimitive()) {
			// We do not want to add anything
		}
		else if (c.isInterface()) {
			classDesc.append(" interface");
		}
		else {
			classDesc.append(" class");
		}

		// Append simple name
		classDesc.append(" " + simpleName);

		// Append super class
		Class superClass = c.getSuperclass();
		if (superClass != null) {
			String superClassSimpleName = superClass.getSimpleName();
			classDesc.append(" extends " + superClassSimpleName);
		}

		// Append Interfaces
		String interfaces = ClassReflection.getClassInterfaces(c);
		if (interfaces != null) {
			classDesc.append(" implements " + interfaces);
		}

		return classDesc.toString();
	}

	public static String getClassInterfaces(Class c) {
		Class[] interfaces = c.getInterfaces();
		String interfacesList = null;
		if (interfaces.length > 0) {
			interfacesList = arrayToString(interfaces, ", ");
		}
		return interfacesList;
	}

	/* Converts array of classes to a string separated by a comma */
	public static String arrayToString(Class[] classes) {
		return arrayToString(classes, ",");
	}

	/* Converts array of classes to a string separated by a separator*/
	public static String arrayToString(Class[] classes, 
								    String separator) {
		StringBuffer result = new StringBuffer();
		if (classes.length > 0) {
			result.append(classes[0].getSimpleName());
			for (int i = 1; i < classes.length; i++) {
				result.append(separator);
				result.append(classes[i].getSimpleName());
			}
		}
		return result.toString();
	}
}
