// ConstructorReflection.java
package com.jdojo.chapter5;

import java.lang.reflect.Constructor;
import java.lang.reflect.Modifier;

public class ConstructorReflection {
	public static void main(String[] args) {
		String className = "com.jdojo.chapter5.Person";

		try {
			Class c = Class.forName(className);

			// Get the declared constructors
			String constructorsDesciption = 
					getDeclaredConstructorsList(c, ";\n");
			System.out.println("Declared Constructors for - " + 
			                    className);
			System.out.println(constructorsDesciption);

			// Get the accessible public constructors
			constructorsDesciption = getConstructorsList(c, ";\n");
			System.out.println("Constructors for - " + className);
			System.out.println(constructorsDesciption);
		}
		catch (ClassNotFoundException e) {
			System.out.println("Exception:" + e.getMessage());
		}
	}

	public static String getConstructorsDesciption
					(Constructor[] constructors, String separator) {
		StringBuffer constructorsDesc = new StringBuffer();
		for (Constructor m : constructors) {
			// Get the modifiers for the class
			int mod = m.getModifiers();
			String modifiers = Modifier.toString(mod);
			constructorsDesc.append(modifiers);

			// Get the name of the method
			String methodName = m.getName();
			constructorsDesc.append(" " + methodName);

			// Get the parameters of the method
			Class[] paramsType = m.getParameterTypes();
			String paramsList = 
				ClassReflection.arrayToString(paramsType, ", ");
			constructorsDesc.append(" (" + paramsList + ")");

			// Get the Exceptions thrown by method
			Class[] eType = m.getExceptionTypes();
			String eList = ClassReflection.arrayToString(eType, ", ");
			if (eList.length() > 0) {
				constructorsDesc.append(" throws " + eList);
			}

			// Append the separator
			constructorsDesc.append(separator);
		}

		return constructorsDesc.toString();
	}

	public static String getConstructorsList(Class c, String separator){
		Constructor[] constructors = c.getConstructors();
		String constructorsList =
				getConstructorsDesciption(constructors, separator);
		return constructorsList;
	}

	public static String getDeclaredConstructorsList(Class c, 
	                                           String separator) {
		Constructor[] constructors = c.getDeclaredConstructors();
		String constructorsList = 
			getConstructorsDesciption(constructors, separator);
		return constructorsList;
	}
}
