// MethodReflection.java
package com.jdojo.chapter5;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

public class MethodReflection {
	public static void main(String[] args) {
		String className = "com.jdojo.chapter5.Person";

		try {
			Class c = Class.forName(className);

			// Get the declared methods
			String methodsDesciption = 
					getDeclaredMethodsList(c, ";\n");
			System.out.println("Declared Methods for - " + className);
			System.out.println(methodsDesciption);

			// Get the accessible public methods
			methodsDesciption = getMethodsList(c, ";\n");
			System.out.println("Methods for - " + className);
			System.out.println(methodsDesciption);
		}
		catch (ClassNotFoundException e) {
			System.out.println("Exception:" + e.getMessage());
		}
	}

	public static String getMethodsDesciption(Method[] methods,
	                                          String separator) {
		StringBuffer methodsDesc = new StringBuffer();
		for (Method m : methods) {
			// Get the modifiers for the class
			int mod = m.getModifiers();
			String modifiers = Modifier.toString(mod);
			methodsDesc.append(modifiers);

			// Get the method return type
			Class returnType = m.getReturnType();
			String returnTypeName = returnType.getSimpleName();
			methodsDesc.append(" " + returnTypeName);

			// Get the name of the method
			String methodName = m.getName();
			methodsDesc.append(" " + methodName);

			// Get the parameters of the method
			Class[] paramsType = m.getParameterTypes();
			String paramsList = 
				ClassReflection.arrayToString(paramsType, ", ");
			methodsDesc.append(" (" + paramsList + ")");

			// Get the Exceptions thrown by method
			Class[] eType = m.getExceptionTypes();
			String eList = ClassReflection.arrayToString(eType, ", ");
			if (eList.length() > 0) {
				methodsDesc.append(" throws " + eList);
			}

			// Append the separator
			methodsDesc.append(separator);
		}

		return methodsDesc.toString();
}
	

	public static String getMethodsList(Class c, String separator) {
		Method[] methods = c.getMethods();
		String methodsList = getMethodsDesciption(methods, separator);
		return methodsList;
	}

	public static String getDeclaredMethodsList(Class c,
		String separator) {
		Method[] methods = c.getDeclaredMethods();
		String methodsList = getMethodsDesciption(methods, separator);
		return methodsList;
	}
}
