// FileCopy.java
package com.jdojo.chapter8;

import java.io.File;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

public class FileCopy {
	public static void main(String[] args) {
		String source = "luci1.txt";
		String sink = "luci1_copy.txt";
		try {
			copy(source, sink, false);
			System.out.println(source + " has been copied to " + sink);
		}
		catch (IOException e) {
			System.out.println(e.getMessage());
		}
	}

	public static void copy(String source, String sink,
	                        boolean overwrite) throws IOException {
		String msg = "";

		// Perform some error checks - does the source file exist?
		File sourceFile = new File(source);
		if (!sourceFile.exists()) {
			msg = "Source file " + sourceFile.getAbsolutePath() +
			      " does not exist.";
			throw new IOException(msg);
		}

		// Perform error check: Can we overwrite the destination file?
		File sinkFile = new File(sink);
		if (sinkFile.exists() && !overwrite) {
			msg = "File " + sinkFile.getAbsolutePath() 
			      + " already exists.";
			throw new IOException(msg);
		}

		BufferedInputStream bis = null;
		BufferedOutputStream bos = null;
		
		try { 
			bis = new BufferedInputStream(
					new FileInputStream(sourceFile));
			bos = new BufferedOutputStream(
					new FileOutputStream(sinkFile));
			int count = -1;
			byte[] buffer = new byte[1024];
			while ((count = bis.read(buffer)) != -1) {
				bos.write(buffer, 0, count);
			}
		}
		finally {
			// Note that if bos.close() throws exception, bis.close
			// will not get executed. Left like this for simplicity	
			if (bos != null) {
				bos.close();
			}

			if (bis != null) {
				bis.close();
			}
		}
	}
}
