// MutipleSerialization.java
package com.jdojo.chapter8;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

public class MutipleSerialization {
	public static void main(String[] args) {
		String fileName = "personmutable.ser";
		
		// Write the same object twice to the stream
		serialize(fileName);
		
		System.out.println("--------------------------------------");
		
		// Read the two objects written to the stream
		deserialize(fileName);
	}

	public static void serialize(String fileName) {
		// Create three PersonMutable objects
		PersonMutable john = new PersonMutable("John", "Male", 6.7);
			
		File fileObject = new File(fileName);
		ObjectOutputStream oos = null;
	
		try {
			// Create an object of ObjectOutputStream
			oos = new ObjectOutputStream (
					new FileOutputStream(fileObject)
				 );

			// Let us display the objects we have 
			// serialized on the console
			System.out.println("Objects are written to: " + 
			                   fileObject.getAbsolutePath());
		
			// Write john object first time to the stream
			oos.writeObject(john);
			System.out.println(john); // display what we wrote
		
			// Change john object's name and height
			john.setName("John Jacobs");
			john.setHeight(6.9);
		
			// Write john object again with changed name and height
			oos.writeObject(john);
			System.out.println(john); // display what we wrote again	
			// object output stream is closed in the finally block
		}
		catch(IOException e1) {
			e1.printStackTrace();
		}
		finally {
			if (oos != null) {
				try {
					oos.close();
				}
				catch(IOException e2) {
					e2.printStackTrace();
				}
			}
		}
	}

	public static void deserialize(String fileName) {
		// personmutable.ser file must exist in the current directory	
		File fileObject = new File(fileName);
		ObjectInputStream ois = null;
		
		try {
			// Create an object of ObjectOutputStream 		 
			ois = new ObjectInputStream (
					new FileInputStream(fileObject)
			      );

			// Read (or deserialize) two objects
			// we wrote in serialize() method
			PersonMutable john1 = (PersonMutable)ois.readObject();
			PersonMutable john2 = (PersonMutable)ois.readObject();

			// Display the objects 
			System.out.println("Two objects are read from " + 
			                   fileObject.getAbsolutePath());
			System.out.println(john1);
			System.out.println(john2); 		
			// object input stream is closed in the finally block  			
		}
		catch(IOException e1) {
			e1.printStackTrace();
		}
		catch (ClassNotFoundException e2) {
			e2.printStackTrace();
		}		
		finally {
			if (ois != null) {
				try {
					ois.close();
				}
				catch(IOException e3) {
					e3.printStackTrace();
				}
			}
		}
	} 
}
