// PersonName.java
package com.jdojo.chapter8;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.io.Serializable;

public class PersonName implements Serializable {
	// Included SUID 
	private static final long serialVersionUID = 7698942876187982839L;

	private String firstName = "Unknown";
	private String lastName = "Unknown";

	/* Declare the serializable field. “name” field is not present 
	   in this class. Still we have included it below so that this
	   class version is backward compatible for serialization. See
	   readObject() and writeObject() methods for details
	*/
	private static final ObjectStreamField serialPersistentFields[] =
					{new ObjectStreamField("name", String.class)};
		
	public PersonName(String name) {
		// Split the name. Error check not included
		String[] names = name.split(" ");
		firstName = names[0];
		lastName = names[1];
	}

	public PersonName(String firstName, String lastName) {
		this.firstName = firstName;
		this.lastName = lastName;
	}

	// Made changes to this method to use new instance variables
	public String toString() {
		return this.firstName + " " + this.lastName;
	}

	private void readObject(ObjectInputStream ois) 
				throws IOException, ClassNotFoundException { 
		// Get the fields from the stream
		ObjectInputStream.GetField fields = ois.readFields();
		
		// Get the name field's value from the stream
		String name = (String)fields.get("name", null);

		// We have omitted error check
		String[] names = name.split(" ");
		this.firstName = names[0];
		this.lastName = names[1];
	}

	private void writeObject(ObjectOutputStream os) throws IOException {
		// Concatenate first and last names and save it in name field
		ObjectOutputStream.PutField fields = os.putFields();
		String name = this.firstName + " " + this.lastName;
		fields.put("name", name);  
		os.writeFields(); 
	}
}
