// PipedStreamTest.java
package com.jdojo.chapter8;

import java.io.PipedInputStream;
import java.io.PipedOutputStream;

public class PipedStreamTest {
	public static void main(String[] args) throws Exception {
		// Create and connect piped input and output streams
		PipedInputStream pis = new PipedInputStream();
		PipedOutputStream pos = new PipedOutputStream();		
		pos.connect(pis);
		
		// Generate and read data in separate threads
		produceData(pos);
		consumeData(pis);
	}

	public static void produceData(PipedOutputStream pos) {
		class Producer extends Thread {
			private PipedOutputStream pos;
			
			Producer(PipedOutputStream pos) {
				this.pos = pos;
			}
			
			public void run() {
				try {
					writeData();
				}
				catch(Exception e) {
					e.printStackTrace();
				}
			}
				
			private void writeData() throws Exception {
				for (int i = 1; i <= 50; i++) {
					pos.write((byte)i);
					pos.flush();
					System.out.println("Writing: " + i);
					Thread.sleep(1000);
				}
				pos.close();
			}
		} // end of local inner class

		// Start a new thread to produce and write numbers to 
		// the piped output stream
		Producer p = new Producer(pos);
		p.start();
	}

	public static void consumeData(PipedInputStream pis) {
		class Consumer extends Thread {
			private PipedInputStream pis;

			Consumer(PipedInputStream pis) {
				this.pis = pis;
			}

			public void run() {
				try {
					readData();
				} 
				catch (Exception e) {
					e.printStackTrace(); 
				}
			} 

			private void readData() throws Exception {
				int num = -1;
				while ((num = pis.read()) != -1) { 
					System.out.println("Reading: " + num);
				}
			}
		} // end of local inner class

		// Start a thread to read the numbers generated 
		// by the producer thread
		Consumer c = new Consumer(pis);
		c.start();
	}
}
