// StreamTokenTest.java
package com.jdojo.chapter8;

import java.io.StreamTokenizer;
import static java.io.StreamTokenizer.*;
import java.io.StringReader;
import java.io.IOException;
import java.io.Reader;
import java.util.TreeMap;

public class StreamTokenTest {
	public static void main(String[] args) throws Exception{
		String str = "This is a test, 200.89 which is simple 50";
		StringReader sr = new StringReader(str);  
		StreamTokenizer st = new StreamTokenizer(sr);

		try {
			while (st.nextToken() != TT_EOF) {
				switch (st.ttype) {
					case TT_WORD: // a word has been read
						System.out.println("String value: " + 
						                   st.sval);
						break;
					case TT_NUMBER: // a number has been read
						System.out.println("Number value: " + 
						                   st.nval);
						break;
				}
			}
		}
		catch(IOException e) {
			e.printStackTrace();
		}
	}

	public static TreeMap getWordCount(Reader reader) 
									throws Exception {
		TreeMap<String, Long> hm = new TreeMap<String, Long>();
		StreamTokenizer st = new StreamTokenizer(reader);
		String token = null;
		while (st.nextToken() != TT_EOF) {
			switch (st.ttype) {
				case TT_WORD: // a word has been read
					token = st.sval;
					break;
				case TT_NUMBER: // a number has been read
					token = String.valueOf(st.nval);
					break;
			}

			if (hm.containsKey(token)) {
				// Token is already there
				long tokenCount = hm.get(token);
				tokenCount++;
				hm.put(token, tokenCount);
			}
			else {
				hm.put(token, 1L);
			}
		}
		return hm;
	}
}
