// UnzipUtility.java
package com.jdojo.chapter9;

import java.util.zip.ZipEntry;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.util.zip.ZipInputStream;

public class UnzipUtility {
	public static void main(String[] args) {
		String zipFileName = "ziptest.zip";
		String unzipdirectory = "extracted";
		unzip(zipFileName, unzipdirectory);
	}

	public static void unzip(String zipFileName, String unzipdir) {
		ZipInputStream zis = null;
		try {
			// Create a zip input stream by wrapping a
			// BufferedInputStream for a better speed
			zis = new ZipInputStream(
					new BufferedInputStream(
						 new FileInputStream(zipFileName)
					)
				 );

			// Read each entry from the ZIP file
			ZipEntry entry = null;
			while((entry = zis.getNextEntry()) != null) {
				// Add the content of the zip entry to the ZIP file
				extractEntryContent(zis, entry, unzipdir);
			}

			System.out.println("ZIP file's contents have been" + 
			                   " extracted to " +  unzipdir + 
			                   " directory.");
			// zip input stream is closed in the finally block
		}
		catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}
		catch (IOException e2) {
			e2.printStackTrace();
		}
		finally {
			// Let us close the zip input stream
			if (zis != null) {
				try {
					zis.close();
				}
				catch(IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public static void extractEntryContent(ZipInputStream zis,
	                                       ZipEntry entry,
	                                       String unzipdir)
					throws IOException, FileNotFoundException {

		String entryFileName = entry.getName();
		String entryPath = unzipdir + "/" + entryFileName;

		// Create the entry file by creating any necessary directories
		createFile(entryPath);

		// Create an output stream to extract the content of the zip 
		// entry and write to the new file
		BufferedOutputStream bos = new BufferedOutputStream (
									new FileOutputStream(entryPath)
								);

		byte[] buffer = new byte[1024];
		int count = -1;
		while((count = zis.read(buffer)) != -1) {
			 bos.write(buffer, 0, count);
		}

		bos.close();
	}

	public static void createFile(String filePath) throws IOException {
		File file = new File(filePath);
		File parent = file.getParentFile();

		// Create all parent directories if they do not exist
		if (!parent.exists()) {
			parent.mkdirs();
		}

		file.createNewFile();
	}
}
