// ZipUtility.java
package com.jdojo.chapter9;

import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.util.zip.Deflater;

public class ZipUtility {
	public static void main(String[] args) {
		// we want to create a ziptest.zip file in the current 
		// directory. We want to add two files to this zip file.
		// Both file paths are relative to the current directory
		String zipFileName = "ziptest.zip";
		String[] entries = new String[2];
		entries[0] = "test1.txt";
		entries[1] = "notes\\test2.txt";
		zip(zipFileName, entries);
	}

	public static void zip(String zipFileName, String[] zipEntries) {
		// Get the current directory for later use
		String currentDirectory = System.getProperty("user.dir");


		ZipOutputStream zos = null;
		try {
			// Create a zip output stream by wrapping a 
			// BufferedOutputStream for better speed 
			zos = new ZipOutputStream(
					new BufferedOutputStream(
						new FileOutputStream(zipFileName)
					)
				 );

			// Set the compression level to best compression
			zos.setLevel(Deflater.BEST_COMPRESSION);

			// Add each entry to ZIP file
			for (int i = 0; i < zipEntries.length; i++) {
				// Make sure the entry file exists
				File entryFile = new File(zipEntries[i]);
				if (!entryFile.exists()) {
					System.out.println("Entry file " +
					                   entryFile.getAbsolutePath() + 
					                   " does not exist");
					System.out.println("Aborted processing.");
					return;
				}
		
				// Create a ZipEntry object
				ZipEntry ze = new ZipEntry(zipEntries[i]);
		
				// Add zip entry object to ZIP file
				zos.putNextEntry(ze);
		
				// Add content of the zip entry to the ZIP file
				addEntryContent(zos, zipEntries[i]);

				// We are done working with the current entry
				zos.closeEntry();
			}		

			System.out.println("Output has been written to: " + 
			                   zipFileName + " in " +
			                   currentDirectory + " directory ");
			// Zip output stream is closed in the finally block
		}
		catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}
		catch (IOException e2) {
			e2.printStackTrace();
		}
		finally {
			// Let us close the zip output stream
			if (zos != null) {
				try {
					zos.close();
				}
				catch(IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public static void addEntryContent(ZipOutputStream zos, 
	                                   String entryFileName) 
					throws IOException, FileNotFoundException {

		// Create an input stream to read data from the entry file
		BufferedInputStream bis = new BufferedInputStream (
								   new FileInputStream(entryFileName)
							  );
		
		byte[] buffer = new byte[1024];
		int count = -1;
		while((count = bis.read(buffer)) != -1) {
			zos.write(buffer, 0, count);
		}
		bis.close(); 
	}
}
